﻿#include "pch.h"
#include "resource.h"

HWND _hwnd = NULL;
cv::Mat _cv_mat;
unsigned int _bi_pocet_bitu = 0;
cv::VideoCapture _kamera;
unsigned int _sirka_videa = 0;
unsigned int _vyska_videa = 0;
unsigned int _obr_za_sekundu = 0;

inline const wchar_t* trida_hlavni() noexcept
{
	return L"opencv_winapi";
}

inline void __declspec(noreturn) kriticke_ukonceni() noexcept
{
	_CrtDbgBreak();
	FatalAppExit(0, L"Došlo k závažné chybě! Aplikace bude ukončena...");
}

void wm_paint() noexcept
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(_hwnd, &ps);
	RECT rect;
	GetClientRect(_hwnd, &rect);
	BITMAPINFO bmi = { 0 };
	bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biWidth = _cv_mat.cols;
	bmi.bmiHeader.biHeight = _cv_mat.rows * -1;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = _bi_pocet_bitu;
	StretchDIBits(hdc, 0, 0, _cv_mat.cols, _cv_mat.rows,
		0, 0, _cv_mat.cols, _cv_mat.rows,
		(RGBTRIPLE*)_cv_mat.data,
		&bmi, DIB_RGB_COLORS, SRCCOPY);
	EndPaint(_hwnd, &ps);
}

void wm_create(HWND hwnd) noexcept
{
	SetTimer(hwnd, 1, 1000 / _obr_za_sekundu, nullptr);
}

// obsluha zprávy WM_TIMER
void wm_timer() noexcept
{
	if (!_kamera.read(_cv_mat))
	{
		PostMessage(_hwnd, WM_CLOSE, 0, 0);
		return;
	}
	SYSTEMTIME st;
	GetLocalTime(&st);
	char sz_cas[50];
	sprintf_s(sz_cas, sizeof(sz_cas), "%d:%.2d:%.2d", st.wHour, st.wMinute, st.wSecond);
	cv::putText(_cv_mat, sz_cas, cv::Point(20, _vyska_videa - 40), CV_FONT_HERSHEY_PLAIN, 2,
		CV_RGB(255, 255,0), 2);
	RedrawWindow(_hwnd, nullptr, nullptr, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);
}

LRESULT CALLBACK window_proc(HWND hwnd, UINT zprava, WPARAM wparam, LPARAM lparam) noexcept
{
	switch (zprava)
	{
	case WM_COMMAND:
		switch (LOWORD(wparam))
		{
		case ID_APLIKACE_KONEC:
			PostMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		}
		break;
	case WM_CREATE:
		wm_create(hwnd);
		break;
	case WM_CLOSE:
		KillTimer(hwnd, 1);
		return DefWindowProc(hwnd, zprava, wparam, lparam);
	case WM_TIMER:
		wm_timer();
		break;
	case WM_PAINT:
		if (!_cv_mat.empty())
			wm_paint();
		else
			return DefWindowProc(hwnd, zprava, wparam, lparam);
		break;
	case WM_ERASEBKGND:
		if (!_cv_mat.empty())
			return 0;
		else
			return DefWindowProc(hwnd, zprava, wparam, lparam);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hwnd, zprava, wparam, lparam);
	}
	return 0;
}

int APIENTRY wWinMain(_In_ HINSTANCE hinstance, _In_opt_ HINSTANCE, _In_ LPWSTR, _In_ int)
{
	if (!_kamera.open(0 + cv::CAP_DSHOW))
	{
		MessageBox(NULL, L"Nepodařilo se spustit žádnou kameru", L"Sorry jako...", MB_ICONERROR);
		return 0;
	}
	if (!_kamera.read(_cv_mat))
		return 0;
	if (!_kamera.set(CAP_PROP_FPS, 25.0))
		if (!_kamera.set(CAP_PROP_FPS, 15.0))
		{
			MessageBox(NULL, L"Tahle kamera je na 2 věci...", L"Sorry jako...", MB_ICONERROR);
			return 0;
		}
	_bi_pocet_bitu = (int)((_cv_mat.dataend - _cv_mat.datastart) / (_cv_mat.cols * _cv_mat.rows) * 8);
	_sirka_videa = (unsigned int)_kamera.get(CAP_PROP_FRAME_WIDTH);
	_vyska_videa = (unsigned int)_kamera.get(CAP_PROP_FRAME_HEIGHT);
	_obr_za_sekundu = (unsigned int)_kamera.get(CAP_PROP_FPS);

	WNDCLASSEXW wcex;
	memset(&wcex, 0, sizeof(wcex));
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = window_proc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = hinstance;
	wcex.hIcon = LoadIcon(hinstance, MAKEINTRESOURCE(IDI_HLAVNI));
	wcex.hCursor = LoadCursor(nullptr, IDC_ARROW);
	wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wcex.lpszClassName = trida_hlavni();
	wcex.lpszMenuName = MAKEINTRESOURCEW(IDR_HLAVNI);
	wcex.hIconSm = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_HLAVNI));
	if (!RegisterClassExW(&wcex))
		kriticke_ukonceni();
	_hwnd = CreateWindowEx(0, trida_hlavni(), L"OpenCV ve WinAPI", WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, nullptr, nullptr, hinstance, nullptr);
	if (!_hwnd)
		kriticke_ukonceni();
	RECT rect = { 0, 0, (LONG)_sirka_videa, (LONG)_vyska_videa };
	AdjustWindowRect(&rect, WS_SYSMENU | WS_SIZEBOX, TRUE);
	SetWindowPos(_hwnd, NULL, 0, 0, rect.right - rect.left,
		rect.bottom - rect.top, SWP_NOMOVE | SWP_NOZORDER);
	ShowWindow(_hwnd, SW_SHOW);
	UpdateWindow(_hwnd);
	MSG msg;
	while (GetMessage(&msg, nullptr, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return (int)msg.wParam;
}

